import type { AgentMessage } from '../../agent/AgentMessage';
import type { AcceptOfferOptions, AcceptProposalOptions, AcceptRequestOptions, NegotiateOfferOptions, NegotiateProposalOptions, OfferCredentialOptions, ProposeCredentialOptions } from './CredentialsModuleOptions';
import type { CredentialExchangeRecord } from './repository/CredentialExchangeRecord';
import type { CredentialService } from './services/CredentialService';
import { AgentConfig } from '../../agent/AgentConfig';
import { MessageSender } from '../../agent/MessageSender';
import { DidCommMessageRepository } from '../../storage/didcomm/DidCommMessageRepository';
import { ConnectionService } from '../connections/services';
import { MediationRecipientService } from '../routing';
import { CredentialProtocolVersion } from './CredentialProtocolVersion';
import { V1CredentialService } from './protocol/v1/V1CredentialService';
import { V2CredentialService } from './protocol/v2/V2CredentialService';
import { CredentialRepository } from './repository/CredentialRepository';
export interface CredentialsModule {
    proposeCredential(options: ProposeCredentialOptions): Promise<CredentialExchangeRecord>;
    acceptProposal(options: AcceptProposalOptions): Promise<CredentialExchangeRecord>;
    negotiateProposal(options: NegotiateProposalOptions): Promise<CredentialExchangeRecord>;
    offerCredential(options: OfferCredentialOptions): Promise<CredentialExchangeRecord>;
    acceptOffer(options: AcceptOfferOptions): Promise<CredentialExchangeRecord>;
    declineOffer(credentialRecordId: string): Promise<CredentialExchangeRecord>;
    negotiateOffer(options: NegotiateOfferOptions): Promise<CredentialExchangeRecord>;
    createOutOfBandOffer(options: OfferCredentialOptions): Promise<{
        message: AgentMessage;
        credentialRecord: CredentialExchangeRecord;
    }>;
    acceptRequest(options: AcceptRequestOptions): Promise<CredentialExchangeRecord>;
    acceptCredential(credentialRecordId: string): Promise<CredentialExchangeRecord>;
    getAll(): Promise<CredentialExchangeRecord[]>;
    getById(credentialRecordId: string): Promise<CredentialExchangeRecord>;
    findById(credentialRecordId: string): Promise<CredentialExchangeRecord | null>;
    deleteById(credentialRecordId: string): Promise<void>;
}
export declare class CredentialsModule implements CredentialsModule {
    private connectionService;
    private messageSender;
    private credentialRepository;
    private agentConfig;
    private didCommMessageRepo;
    private v1Service;
    private v2Service;
    private mediatorRecipientService;
    private logger;
    private serviceMap;
    constructor(messageSender: MessageSender, connectionService: ConnectionService, agentConfig: AgentConfig, credentialRepository: CredentialRepository, mediationRecipientService: MediationRecipientService, didCommMessageRepository: DidCommMessageRepository, v1Service: V1CredentialService, v2Service: V2CredentialService);
    getService(protocolVersion: CredentialProtocolVersion): CredentialService;
}
