"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.V1CredentialService = void 0;
const tsyringe_1 = require("tsyringe");
const AgentConfig_1 = require("../../../../agent/AgentConfig");
const Dispatcher_1 = require("../../../../agent/Dispatcher");
const EventEmitter_1 = require("../../../../agent/EventEmitter");
const ServiceDecorator_1 = require("../../../../decorators/service/ServiceDecorator");
const error_1 = require("../../../../error");
const storage_1 = require("../../../../storage");
const attachment_1 = require("../../../../utils/attachment");
const common_1 = require("../../../common");
const services_1 = require("../../../connections/services");
const routing_1 = require("../../../routing");
const CredentialAutoAcceptType_1 = require("../../CredentialAutoAcceptType");
const CredentialEvents_1 = require("../../CredentialEvents");
const CredentialProtocolVersion_1 = require("../../CredentialProtocolVersion");
const CredentialResponseCoordinator_1 = require("../../CredentialResponseCoordinator");
const CredentialState_1 = require("../../CredentialState");
const CredentialUtils_1 = require("../../CredentialUtils");
const errors_1 = require("../../errors");
const IndyCredentialFormatService_1 = require("../../formats/indy/IndyCredentialFormatService");
const repository_1 = require("../../repository");
const services_2 = require("../../services");
const V1CredentialPreview_1 = require("./V1CredentialPreview");
const handlers_1 = require("./handlers");
const messages_1 = require("./messages");
let V1CredentialService = class V1CredentialService extends services_2.CredentialService {
    constructor(connectionService, didCommMessageRepository, agentConfig, mediationRecipientService, dispatcher, eventEmitter, credentialRepository, formatService, revocationService) {
        super(credentialRepository, eventEmitter, dispatcher, agentConfig, mediationRecipientService, didCommMessageRepository, revocationService);
        this.connectionService = connectionService;
        this.formatService = formatService;
    }
    /**
     * Create a {@link ProposeCredentialMessage} not bound to an existing credential exchange.
     * To create a proposal as response to an existing credential exchange, use {@link createProposalAsResponse}.
     *
     * @param proposal The object containing config options
     * @returns Object containing proposal message and associated credential record
     *
     */
    async createProposal(proposal) {
        var _a, _b, _c, _d, _e, _f, _g;
        const connection = await this.connectionService.getById(proposal.connectionId);
        connection.assertReady();
        let credentialProposal;
        const credPropose = (_a = proposal.credentialFormats.indy) === null || _a === void 0 ? void 0 : _a.payload;
        if ((_b = proposal.credentialFormats.indy) === null || _b === void 0 ? void 0 : _b.attributes) {
            credentialProposal = new V1CredentialPreview_1.V1CredentialPreview({ attributes: (_c = proposal.credentialFormats.indy) === null || _c === void 0 ? void 0 : _c.attributes });
        }
        const config = {
            credentialProposal: credentialProposal,
            credentialDefinitionId: credPropose === null || credPropose === void 0 ? void 0 : credPropose.credentialDefinitionId,
            linkedAttachments: (_d = proposal.credentialFormats.indy) === null || _d === void 0 ? void 0 : _d.linkedAttachments,
            schemaId: credPropose === null || credPropose === void 0 ? void 0 : credPropose.schemaId,
        };
        const options = Object.assign({}, config);
        const { attachment: filtersAttach } = this.formatService.createProposal(proposal);
        if (!filtersAttach) {
            throw new error_1.AriesFrameworkError('Missing filters attach in Proposal');
        }
        options.attachments = [];
        (_e = options.attachments) === null || _e === void 0 ? void 0 : _e.push(filtersAttach);
        // Create message
        const message = new messages_1.V1ProposeCredentialMessage(options !== null && options !== void 0 ? options : {});
        // Create record
        const credentialRecord = new repository_1.CredentialExchangeRecord({
            connectionId: connection.id,
            threadId: message.threadId,
            state: CredentialState_1.CredentialState.ProposalSent,
            linkedAttachments: (_f = config === null || config === void 0 ? void 0 : config.linkedAttachments) === null || _f === void 0 ? void 0 : _f.map((linkedAttachment) => linkedAttachment.attachment),
            credentialAttributes: (_g = message.credentialProposal) === null || _g === void 0 ? void 0 : _g.attributes,
            autoAcceptCredential: config === null || config === void 0 ? void 0 : config.autoAcceptCredential,
            protocolVersion: CredentialProtocolVersion_1.CredentialProtocolVersion.V1,
            credentials: [],
        });
        // Set the metadata
        credentialRecord.metadata.set(repository_1.CredentialMetadataKeys.IndyCredential, {
            schemaId: options.schemaId,
            credentialDefinitionId: options.credentialDefinitionId,
        });
        await this.credentialRepository.save(credentialRecord);
        await this.didCommMessageRepository.saveAgentMessage({
            agentMessage: message,
            role: storage_1.DidCommMessageRole.Sender,
            associatedRecordId: credentialRecord.id,
        });
        this.eventEmitter.emit({
            type: CredentialEvents_1.CredentialEventTypes.CredentialStateChanged,
            payload: {
                credentialRecord,
                previousState: null,
            },
        });
        return { credentialRecord, message };
    }
    /**
     * Processing an incoming credential message and create a credential offer as a response
     * @param proposal The object containing config options
     * @param credentialRecord the credential exchange record for this proposal
     * @returns Object containing proposal message and associated credential record
     */
    async acceptProposal(options, credentialRecord) {
        var _a, _b;
        const proposalCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1ProposeCredentialMessage,
        });
        if (!(proposalCredentialMessage === null || proposalCredentialMessage === void 0 ? void 0 : proposalCredentialMessage.credentialProposal)) {
            throw new error_1.AriesFrameworkError(`Credential record with id ${options.credentialRecordId} is missing required credential proposal`);
        }
        if (!options.credentialFormats) {
            throw new error_1.AriesFrameworkError('Missing credential formats in V1 acceptProposal');
        }
        const credentialDefinitionId = (_b = (_a = options.credentialFormats.indy) === null || _a === void 0 ? void 0 : _a.credentialDefinitionId) !== null && _b !== void 0 ? _b : proposalCredentialMessage.credentialDefinitionId;
        if (!credentialDefinitionId) {
            throw new error_1.AriesFrameworkError('Missing required credential definition id. If credential proposal message contains no credential definition id it must be passed to config.');
        }
        const { message } = await this.createOfferAsResponse(credentialRecord, {
            preview: proposalCredentialMessage.credentialProposal,
            credentialDefinitionId,
            comment: options.comment,
            autoAcceptCredential: options.autoAcceptCredential,
            attachments: credentialRecord.linkedAttachments,
        });
        return { credentialRecord, message };
    }
    /**
     * Negotiate a credential proposal as issuer (by sending a credential offer message) to the connection
     * associated with the credential record.
     *
     * @param credentialOptions configuration for the offer see {@link NegotiateProposalOptions}
     * @param credentialRecord the credential exchange record for this proposal
     * @returns Credential record associated with the credential offer and the corresponding new offer message
     *
     */
    async negotiateProposal(credentialOptions, credentialRecord) {
        var _a, _b, _c, _d;
        if (!credentialRecord.connectionId) {
            throw new error_1.AriesFrameworkError(`No connectionId found for credential record '${credentialRecord.id}'. Connection-less issuance does not support negotiation.`);
        }
        const credentialProposalMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1ProposeCredentialMessage,
        });
        if (!(credentialProposalMessage === null || credentialProposalMessage === void 0 ? void 0 : credentialProposalMessage.credentialProposal)) {
            throw new error_1.AriesFrameworkError(`Credential record with id ${credentialOptions.credentialRecordId} is missing required credential proposal`);
        }
        const credentialDefinitionId = (_b = (_a = credentialOptions.credentialFormats.indy) === null || _a === void 0 ? void 0 : _a.credentialDefinitionId) !== null && _b !== void 0 ? _b : credentialProposalMessage.credentialDefinitionId;
        if (!credentialDefinitionId) {
            throw new error_1.AriesFrameworkError('Missing required credential definition id. If credential proposal message contains no credential definition id it must be passed to config.');
        }
        if (!((_c = credentialOptions === null || credentialOptions === void 0 ? void 0 : credentialOptions.credentialFormats.indy) === null || _c === void 0 ? void 0 : _c.attributes)) {
            throw new error_1.AriesFrameworkError('No proposal attributes in the negotiation options!');
        }
        const newCredentialProposal = new V1CredentialPreview_1.V1CredentialPreview({
            attributes: (_d = credentialOptions === null || credentialOptions === void 0 ? void 0 : credentialOptions.credentialFormats.indy) === null || _d === void 0 ? void 0 : _d.attributes,
        });
        const { message } = await this.createOfferAsResponse(credentialRecord, {
            preview: newCredentialProposal,
            credentialDefinitionId,
            comment: credentialOptions.comment,
            autoAcceptCredential: credentialOptions.autoAcceptCredential,
            attachments: credentialRecord.linkedAttachments,
        });
        return { credentialRecord, message };
    }
    /**
     * Process a received {@link ProposeCredentialMessage}. This will not accept the credential proposal
     * or send a credential offer. It will only create a new, or update the existing credential record with
     * the information from the credential proposal message. Use {@link createOfferAsResponse}
     * after calling this method to create a credential offer.
     *
     * @param messageContext The message context containing a credential proposal message
     * @returns credential record associated with the credential proposal message
     *
     */
    async processProposal(messageContext) {
        var _a;
        let credentialRecord;
        const { message: proposalMessage, connection } = messageContext;
        this.logger.debug(`Processing credential proposal with id ${proposalMessage.id}`);
        try {
            // Credential record already exists
            credentialRecord = await this.getByThreadAndConnectionId(proposalMessage.threadId, connection === null || connection === void 0 ? void 0 : connection.id);
            // Assert
            credentialRecord.assertState(CredentialState_1.CredentialState.OfferSent);
            const proposalCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
                associatedRecordId: credentialRecord.id,
                messageClass: messages_1.V1ProposeCredentialMessage,
            });
            const offerCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
                associatedRecordId: credentialRecord.id,
                messageClass: messages_1.V1OfferCredentialMessage,
            });
            this.connectionService.assertConnectionOrServiceDecorator(messageContext, {
                previousReceivedMessage: proposalCredentialMessage !== null && proposalCredentialMessage !== void 0 ? proposalCredentialMessage : undefined,
                previousSentMessage: offerCredentialMessage !== null && offerCredentialMessage !== void 0 ? offerCredentialMessage : undefined,
            });
            // Update record
            await this.updateState(credentialRecord, CredentialState_1.CredentialState.ProposalReceived);
            await this.didCommMessageRepository.saveAgentMessage({
                agentMessage: proposalMessage,
                role: storage_1.DidCommMessageRole.Receiver,
                associatedRecordId: credentialRecord.id,
            });
        }
        catch (_b) {
            // No credential record exists with thread id
            credentialRecord = new repository_1.CredentialExchangeRecord({
                connectionId: connection === null || connection === void 0 ? void 0 : connection.id,
                threadId: proposalMessage.threadId,
                credentialAttributes: (_a = proposalMessage.credentialProposal) === null || _a === void 0 ? void 0 : _a.attributes,
                state: CredentialState_1.CredentialState.ProposalReceived,
                protocolVersion: CredentialProtocolVersion_1.CredentialProtocolVersion.V1,
                credentials: [],
            });
            credentialRecord.metadata.set(repository_1.CredentialMetadataKeys.IndyCredential, {
                schemaId: proposalMessage.schemaId,
                credentialDefinitionId: proposalMessage.credentialDefinitionId,
            });
            // Assert
            this.connectionService.assertConnectionOrServiceDecorator(messageContext);
            // Save record
            await this.credentialRepository.save(credentialRecord);
            this.eventEmitter.emit({
                type: CredentialEvents_1.CredentialEventTypes.CredentialStateChanged,
                payload: {
                    credentialRecord,
                    previousState: null,
                },
            });
            await this.didCommMessageRepository.saveAgentMessage({
                agentMessage: proposalMessage,
                role: storage_1.DidCommMessageRole.Receiver,
                associatedRecordId: credentialRecord.id,
            });
        }
        return credentialRecord;
    }
    /**
     * Create a {@link OfferCredentialMessage} as response to a received credential proposal.
     * To create an offer not bound to an existing credential exchange, use {@link createOffer}.
     *
     * @param credentialRecord The credential record for which to create the credential offer
     * @param credentialTemplate The credential template to use for the offer
     * @returns Object containing offer message and associated credential record
     *
     */
    async createOfferAsResponse(credentialRecord, credentialTemplate) {
        var _a;
        // Assert
        credentialRecord.assertState(CredentialState_1.CredentialState.ProposalReceived);
        // Create message
        const { credentialDefinitionId, comment, preview, attachments } = credentialTemplate;
        const options = {
            attachId: messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID,
            credentialFormats: {
                indy: {
                    credentialDefinitionId,
                    attributes: preview.attributes,
                },
            },
            protocolVersion: CredentialProtocolVersion_1.CredentialProtocolVersion.V1,
        };
        const { attachment: offersAttach } = await this.formatService.createOffer(options);
        if (!offersAttach) {
            throw new error_1.AriesFrameworkError('No offer attachment for credential');
        }
        const credOffer = offersAttach.getDataAsJson();
        if (!offersAttach) {
            throw new error_1.AriesFrameworkError('Missing offers attach in Offer');
        }
        const offerMessage = new messages_1.V1OfferCredentialMessage({
            comment,
            offerAttachments: [offersAttach],
            credentialPreview: preview,
            attachments,
        });
        offerMessage.setThread({
            threadId: credentialRecord.threadId,
        });
        credentialRecord.credentialAttributes = preview.attributes;
        credentialRecord.metadata.set(repository_1.CredentialMetadataKeys.IndyCredential, {
            schemaId: credOffer.schema_id,
            credentialDefinitionId: credOffer.cred_def_id,
        });
        credentialRecord.linkedAttachments = attachments === null || attachments === void 0 ? void 0 : attachments.filter((attachment) => attachment_1.isLinkedAttachment(attachment));
        credentialRecord.autoAcceptCredential =
            (_a = credentialTemplate.autoAcceptCredential) !== null && _a !== void 0 ? _a : credentialRecord.autoAcceptCredential;
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.OfferSent);
        await this.didCommMessageRepository.saveAgentMessage({
            agentMessage: offerMessage,
            role: storage_1.DidCommMessageRole.Sender,
            associatedRecordId: credentialRecord.id,
        });
        return { message: offerMessage, credentialRecord };
    }
    /**
     * Process a received {@link RequestCredentialMessage}. This will not accept the credential request
     * or send a credential. It will only update the existing credential record with
     * the information from the credential request message. Use {@link createCredential}
     * after calling this method to create a credential.
     *
     * @param messageContext The message context containing a credential request message
     * @returns credential record associated with the credential request message
     *
     */
    async negotiateOffer(credentialOptions, credentialRecord) {
        var _a, _b, _c;
        if (!credentialRecord.connectionId) {
            throw new error_1.AriesFrameworkError(`No connectionId found for credential record '${credentialRecord.id}'. Connection-less issuance does not support negotiation.`);
        }
        if (!((_a = credentialOptions.credentialFormats.indy) === null || _a === void 0 ? void 0 : _a.attributes)) {
            throw new error_1.AriesFrameworkError('Missing attributes in V1 Negotiate Offer Options');
        }
        const credentialPreview = new V1CredentialPreview_1.V1CredentialPreview({
            attributes: (_b = credentialOptions.credentialFormats.indy) === null || _b === void 0 ? void 0 : _b.attributes,
        });
        const options = {
            credentialProposal: credentialPreview,
        };
        credentialRecord.assertState(CredentialState_1.CredentialState.OfferReceived);
        // Create message
        const message = new messages_1.V1ProposeCredentialMessage(options !== null && options !== void 0 ? options : {});
        message.setThread({ threadId: credentialRecord.threadId });
        // Update record
        credentialRecord.credentialAttributes = (_c = message.credentialProposal) === null || _c === void 0 ? void 0 : _c.attributes;
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.ProposalSent);
        await this.didCommMessageRepository.saveAgentMessage({
            agentMessage: message,
            role: storage_1.DidCommMessageRole.Sender,
            associatedRecordId: credentialRecord.id,
        });
        return { credentialRecord, message };
    }
    /**
     * Create a {@link OfferCredentialMessage} not bound to an existing credential exchange.
     * To create an offer as response to an existing credential exchange, use {@link V1CredentialService#createOfferAsResponse}.
     *
     * @param credentialOptions The options containing config params for creating the credential offer
     * @returns Object containing offer message and associated credential record
     *
     */
    async createOffer(credentialOptions) {
        var _a, _b, _c, _d, _e;
        if (!credentialOptions.connectionId) {
            throw new error_1.AriesFrameworkError('Connection id missing from offer credential options');
        }
        const connection = await this.connectionService.getById(credentialOptions.connectionId);
        if (!((_a = credentialOptions === null || credentialOptions === void 0 ? void 0 : credentialOptions.credentialFormats.indy) === null || _a === void 0 ? void 0 : _a.attributes) ||
            !((_b = credentialOptions === null || credentialOptions === void 0 ? void 0 : credentialOptions.credentialFormats.indy) === null || _b === void 0 ? void 0 : _b.credentialDefinitionId)) {
            throw new error_1.AriesFrameworkError('Missing properties from OfferCredentialOptions object: cannot create Offer!');
        }
        const preview = new V1CredentialPreview_1.V1CredentialPreview({
            attributes: (_c = credentialOptions.credentialFormats.indy) === null || _c === void 0 ? void 0 : _c.attributes,
        });
        const linkedAttachments = (_d = credentialOptions.credentialFormats.indy) === null || _d === void 0 ? void 0 : _d.linkedAttachments;
        const template = Object.assign(Object.assign({}, credentialOptions), { preview: preview, credentialDefinitionId: (_e = credentialOptions === null || credentialOptions === void 0 ? void 0 : credentialOptions.credentialFormats.indy) === null || _e === void 0 ? void 0 : _e.credentialDefinitionId, linkedAttachments });
        const { credentialRecord, message } = await this.createOfferProcessing(template, connection);
        await this.credentialRepository.save(credentialRecord);
        this.eventEmitter.emit({
            type: CredentialEvents_1.CredentialEventTypes.CredentialStateChanged,
            payload: {
                credentialRecord,
                previousState: null,
            },
        });
        await this.didCommMessageRepository.saveAgentMessage({
            agentMessage: message,
            role: storage_1.DidCommMessageRole.Sender,
            associatedRecordId: credentialRecord.id,
        });
        return { credentialRecord, message };
    }
    /**
     * Process a received {@link OfferCredentialMessage}. This will not accept the credential offer
     * or send a credential request. It will only create a new credential record with
     * the information from the credential offer message. Use {@link createRequest}
     * after calling this method to create a credential request.
     *
     * @param messageContext The message context containing a credential request message
     * @returns credential record associated with the credential offer message
     *
     */
    async processOffer(messageContext) {
        var _a;
        let credentialRecord;
        const { message: offerMessage, connection } = messageContext;
        this.logger.debug(`Processing credential offer with id ${offerMessage.id}`);
        const indyCredentialOffer = offerMessage.indyCredentialOffer;
        if (!indyCredentialOffer) {
            throw new errors_1.CredentialProblemReportError(`Missing required base64 or json encoded attachment data for credential offer with thread id ${offerMessage.threadId}`, { problemCode: errors_1.CredentialProblemReportReason.IssuanceAbandoned });
        }
        try {
            // Credential record already exists
            credentialRecord = await this.getByThreadAndConnectionId(offerMessage.threadId, connection === null || connection === void 0 ? void 0 : connection.id);
            const proposalCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
                associatedRecordId: credentialRecord.id,
                messageClass: messages_1.V1ProposeCredentialMessage,
            });
            const offerCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
                associatedRecordId: credentialRecord.id,
                messageClass: messages_1.V1OfferCredentialMessage,
            });
            // Assert
            credentialRecord.assertState(CredentialState_1.CredentialState.ProposalSent);
            this.connectionService.assertConnectionOrServiceDecorator(messageContext, {
                previousReceivedMessage: offerCredentialMessage !== null && offerCredentialMessage !== void 0 ? offerCredentialMessage : undefined,
                previousSentMessage: proposalCredentialMessage !== null && proposalCredentialMessage !== void 0 ? proposalCredentialMessage : undefined,
            });
            credentialRecord.linkedAttachments = (_a = offerMessage.appendedAttachments) === null || _a === void 0 ? void 0 : _a.filter(attachment_1.isLinkedAttachment);
            const attachment = offerCredentialMessage
                ? offerCredentialMessage.getAttachmentById(messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID)
                : undefined;
            if (attachment) {
                await this.formatService.processOffer(attachment, credentialRecord);
            }
            credentialRecord.metadata.set(repository_1.CredentialMetadataKeys.IndyCredential, {
                schemaId: indyCredentialOffer.schema_id,
                credentialDefinitionId: indyCredentialOffer.cred_def_id,
            });
            await this.updateState(credentialRecord, CredentialState_1.CredentialState.OfferReceived);
            await this.didCommMessageRepository.saveAgentMessage({
                agentMessage: offerMessage,
                role: storage_1.DidCommMessageRole.Receiver,
                associatedRecordId: credentialRecord.id,
            });
        }
        catch (_b) {
            // No credential record exists with thread id
            credentialRecord = new repository_1.CredentialExchangeRecord({
                connectionId: connection === null || connection === void 0 ? void 0 : connection.id,
                threadId: offerMessage.id,
                credentialAttributes: offerMessage.credentialPreview.attributes,
                state: CredentialState_1.CredentialState.OfferReceived,
                protocolVersion: CredentialProtocolVersion_1.CredentialProtocolVersion.V1,
                credentials: [],
            });
            credentialRecord.metadata.set(repository_1.CredentialMetadataKeys.IndyCredential, {
                schemaId: indyCredentialOffer.schema_id,
                credentialDefinitionId: indyCredentialOffer.cred_def_id,
            });
            // Assert
            this.connectionService.assertConnectionOrServiceDecorator(messageContext);
            // Save in repository
            await this.credentialRepository.save(credentialRecord);
            await this.didCommMessageRepository.saveAgentMessage({
                agentMessage: offerMessage,
                role: storage_1.DidCommMessageRole.Receiver,
                associatedRecordId: credentialRecord.id,
            });
            this.eventEmitter.emit({
                type: CredentialEvents_1.CredentialEventTypes.CredentialStateChanged,
                payload: {
                    credentialRecord,
                    previousState: null,
                },
            });
        }
        return credentialRecord;
    }
    async createOfferProcessing(credentialTemplate, connectionRecord) {
        // Assert
        connectionRecord === null || connectionRecord === void 0 ? void 0 : connectionRecord.assertReady();
        // Create message
        const { credentialDefinitionId, comment, preview, linkedAttachments } = credentialTemplate;
        // Create and link credential to attachment
        const credentialPreview = linkedAttachments
            ? CredentialUtils_1.CredentialUtils.createAndLinkAttachmentsToPreview(linkedAttachments, preview)
            : preview;
        const options = {
            attachId: messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID,
            credentialFormats: {
                indy: {
                    credentialDefinitionId,
                    attributes: credentialPreview.attributes,
                },
            },
            protocolVersion: CredentialProtocolVersion_1.CredentialProtocolVersion.V1,
        };
        const { attachment: offersAttach } = await this.formatService.createOffer(options);
        if (!offersAttach) {
            throw new error_1.AriesFrameworkError('Missing offers attach in Offer');
        }
        // Construct offer message
        const offerMessage = new messages_1.V1OfferCredentialMessage({
            comment,
            offerAttachments: [offersAttach],
            credentialPreview,
            attachments: linkedAttachments === null || linkedAttachments === void 0 ? void 0 : linkedAttachments.map((linkedAttachment) => linkedAttachment.attachment),
        });
        // Create record
        const credentialRecord = new repository_1.CredentialExchangeRecord({
            connectionId: connectionRecord === null || connectionRecord === void 0 ? void 0 : connectionRecord.id,
            threadId: offerMessage.id,
            credentialAttributes: credentialPreview.attributes,
            linkedAttachments: linkedAttachments === null || linkedAttachments === void 0 ? void 0 : linkedAttachments.map((linkedAttachments) => linkedAttachments.attachment),
            state: CredentialState_1.CredentialState.OfferSent,
            autoAcceptCredential: credentialTemplate.autoAcceptCredential,
            protocolVersion: CredentialProtocolVersion_1.CredentialProtocolVersion.V1,
            credentials: [],
        });
        const offer = offersAttach.getDataAsJson();
        credentialRecord.metadata.set(repository_1.CredentialMetadataKeys.IndyCredential, {
            credentialDefinitionId: credentialDefinitionId,
            schemaId: offer.schema_id,
        });
        return { message: offerMessage, credentialRecord };
    }
    async createOutOfBandOffer(credentialOptions) {
        var _a, _b, _c;
        if (!((_a = credentialOptions.credentialFormats.indy) === null || _a === void 0 ? void 0 : _a.credentialDefinitionId)) {
            throw new error_1.AriesFrameworkError('Missing credential definition id for out of band credential');
        }
        const v1Preview = new V1CredentialPreview_1.V1CredentialPreview({
            attributes: (_b = credentialOptions.credentialFormats.indy) === null || _b === void 0 ? void 0 : _b.attributes,
        });
        const template = {
            credentialDefinitionId: (_c = credentialOptions.credentialFormats.indy) === null || _c === void 0 ? void 0 : _c.credentialDefinitionId,
            comment: credentialOptions.comment,
            preview: v1Preview,
            autoAcceptCredential: credentialOptions.autoAcceptCredential,
        };
        const { credentialRecord, message } = await this.createOfferProcessing(template);
        // Create and set ~service decorator
        const routing = await this.mediationRecipientService.getRouting();
        message.service = new ServiceDecorator_1.ServiceDecorator({
            serviceEndpoint: routing.endpoints[0],
            recipientKeys: [routing.verkey],
            routingKeys: routing.routingKeys,
        });
        await this.credentialRepository.save(credentialRecord);
        await this.didCommMessageRepository.saveAgentMessage({
            agentMessage: message,
            role: storage_1.DidCommMessageRole.Receiver,
            associatedRecordId: credentialRecord.id,
        });
        this.eventEmitter.emit({
            type: CredentialEvents_1.CredentialEventTypes.CredentialStateChanged,
            payload: {
                credentialRecord,
                previousState: null,
            },
        });
        return { credentialRecord, message };
    }
    /**
     * Create a {@link RequestCredentialMessage} as response to a received credential offer.
     *
     * @param record The credential record for which to create the credential request
     * @param options Additional configuration to use for the credential request
     * @returns Object containing request message and associated credential record
     *
     */
    async createRequest(record, options, holderDid) {
        var _a, _b, _c;
        // Assert credential
        record.assertState(CredentialState_1.CredentialState.OfferReceived);
        const offerCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: record.id,
            messageClass: messages_1.V1OfferCredentialMessage,
        });
        // remove
        if (!offerCredentialMessage) {
            throw new errors_1.CredentialProblemReportError(`Missing required credential offer with thread id ${record.threadId}`, {
                problemCode: errors_1.CredentialProblemReportReason.IssuanceAbandoned,
            });
        }
        const attachment = offerCredentialMessage.getAttachmentById(messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID);
        if (attachment) {
            options.offerAttachment = attachment;
        }
        else {
            throw new error_1.AriesFrameworkError(`Missing data payload in attachment in credential Record ${record.id}`);
        }
        options.attachId = messages_1.INDY_CREDENTIAL_REQUEST_ATTACHMENT_ID;
        const { attachment: requestAttach } = await this.formatService.createRequest(options, record, holderDid);
        if (!requestAttach) {
            throw new error_1.AriesFrameworkError(`Failed to create attachment for request; credential record = ${record.id}`);
        }
        const requestMessage = new messages_1.V1RequestCredentialMessage({
            comment: options === null || options === void 0 ? void 0 : options.comment,
            requestAttachments: [requestAttach],
            attachments: (_a = offerCredentialMessage === null || offerCredentialMessage === void 0 ? void 0 : offerCredentialMessage.appendedAttachments) === null || _a === void 0 ? void 0 : _a.filter((attachment) => attachment_1.isLinkedAttachment(attachment)),
        });
        requestMessage.setThread({ threadId: record.threadId });
        record.autoAcceptCredential = (_b = options === null || options === void 0 ? void 0 : options.autoAcceptCredential) !== null && _b !== void 0 ? _b : record.autoAcceptCredential;
        record.linkedAttachments = (_c = offerCredentialMessage === null || offerCredentialMessage === void 0 ? void 0 : offerCredentialMessage.appendedAttachments) === null || _c === void 0 ? void 0 : _c.filter((attachment) => attachment_1.isLinkedAttachment(attachment));
        await this.updateState(record, CredentialState_1.CredentialState.RequestSent);
        return { message: requestMessage, credentialRecord: record };
    }
    /**
     * Process a received {@link IssueCredentialMessage}. This will not accept the credential
     * or send a credential acknowledgement. It will only update the existing credential record with
     * the information from the issue credential message. Use {@link createAck}
     * after calling this method to create a credential acknowledgement.
     *
     * @param messageContext The message context containing an issue credential message
     *
     * @returns credential record associated with the issue credential message
     *
     */
    async processRequest(messageContext) {
        var _a;
        const { message: requestMessage, connection } = messageContext;
        this.logger.debug(`Processing credential request with id ${requestMessage.id}`);
        const credentialRecord = await this.getByThreadAndConnectionId(requestMessage.threadId, connection === null || connection === void 0 ? void 0 : connection.id);
        const proposalMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1ProposeCredentialMessage,
        });
        const offerMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1OfferCredentialMessage,
        });
        // Assert
        credentialRecord.assertState(CredentialState_1.CredentialState.OfferSent);
        this.connectionService.assertConnectionOrServiceDecorator(messageContext, {
            previousReceivedMessage: proposalMessage !== null && proposalMessage !== void 0 ? proposalMessage : undefined,
            previousSentMessage: offerMessage !== null && offerMessage !== void 0 ? offerMessage : undefined,
        });
        const requestOptions = {
            connectionId: (_a = messageContext.connection) === null || _a === void 0 ? void 0 : _a.id,
        };
        await this.formatService.processRequest(requestOptions, credentialRecord);
        this.logger.trace('Credential record found when processing credential request', credentialRecord);
        await this.didCommMessageRepository.saveAgentMessage({
            agentMessage: requestMessage,
            role: storage_1.DidCommMessageRole.Receiver,
            associatedRecordId: credentialRecord.id,
        });
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.RequestReceived);
        return credentialRecord;
    }
    /**
     * Create a {@link IssueCredentialMessage} as response to a received credential request.
     *
     * @param record The credential record for which to create the credential
     * @param options Additional configuration to use for the credential
     * @returns Object containing issue credential message and associated credential record
     *
     */
    async createCredential(record, options) {
        var _a, _b, _c;
        // Assert
        record.assertState(CredentialState_1.CredentialState.RequestReceived);
        const offerMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: record.id,
            messageClass: messages_1.V1OfferCredentialMessage,
        });
        const requestMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: record.id,
            messageClass: messages_1.V1RequestCredentialMessage,
        });
        // Assert offer message
        if (!offerMessage) {
            throw new error_1.AriesFrameworkError(`Missing credential offer for credential exchange with thread id ${record.threadId}`);
        }
        if (!requestMessage) {
            throw new error_1.AriesFrameworkError(`Missing request message in credential Record ${record.id}`);
        }
        let offerAttachment;
        if (offerMessage) {
            offerAttachment = offerMessage.getAttachmentById(messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID);
        }
        else {
            throw new error_1.AriesFrameworkError(`Missing data payload in attachment in credential Record ${record.id}`);
        }
        const requestAttachment = requestMessage.getAttachmentById(messages_1.INDY_CREDENTIAL_REQUEST_ATTACHMENT_ID);
        if (!requestAttachment) {
            throw new error_1.AriesFrameworkError('Missing requestAttachment in v1 createCredential');
        }
        options.attachId = messages_1.INDY_CREDENTIAL_ATTACHMENT_ID;
        // Assert credential attributes
        const credentialAttributes = record.credentialAttributes;
        if (!credentialAttributes) {
            throw new errors_1.CredentialProblemReportError(`Missing required credential attribute values on credential record with id ${record.id}`, { problemCode: errors_1.CredentialProblemReportReason.IssuanceAbandoned });
        }
        const { attachment: credentialsAttach } = await this.formatService.createCredential(options, record, requestAttachment, offerAttachment);
        if (!credentialsAttach) {
            throw new error_1.AriesFrameworkError(`Failed to create attachment for request; credential record = ${record.id}`);
        }
        const issueMessage = new messages_1.V1IssueCredentialMessage({
            comment: options === null || options === void 0 ? void 0 : options.comment,
            credentialAttachments: [credentialsAttach],
            attachments: ((_a = offerMessage === null || offerMessage === void 0 ? void 0 : offerMessage.appendedAttachments) === null || _a === void 0 ? void 0 : _a.filter((attachment) => attachment_1.isLinkedAttachment(attachment))) ||
                ((_b = requestMessage === null || requestMessage === void 0 ? void 0 : requestMessage.appendedAttachments) === null || _b === void 0 ? void 0 : _b.filter((attachment) => attachment_1.isLinkedAttachment(attachment))),
        });
        issueMessage.setThread({
            threadId: record.threadId,
        });
        issueMessage.setPleaseAck();
        record.autoAcceptCredential = (_c = options === null || options === void 0 ? void 0 : options.autoAcceptCredential) !== null && _c !== void 0 ? _c : record.autoAcceptCredential;
        await this.updateState(record, CredentialState_1.CredentialState.CredentialIssued);
        return { message: issueMessage, credentialRecord: record };
    }
    /**
     * Process an incoming {@link IssueCredentialMessage}
     *
     * @param messageContext The message context containing a credential acknowledgement message
     * @returns credential record associated with the credential acknowledgement message
     *
     */
    async processCredential(messageContext) {
        const { message: issueMessage, connection } = messageContext;
        this.logger.debug(`Processing credential with id ${issueMessage.id}`);
        const credentialRecord = await this.getByThreadAndConnectionId(issueMessage.threadId, connection === null || connection === void 0 ? void 0 : connection.id);
        const requestCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1RequestCredentialMessage,
        });
        const offerCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1OfferCredentialMessage,
        });
        // Assert
        credentialRecord.assertState(CredentialState_1.CredentialState.RequestSent);
        this.connectionService.assertConnectionOrServiceDecorator(messageContext, {
            previousReceivedMessage: offerCredentialMessage !== null && offerCredentialMessage !== void 0 ? offerCredentialMessage : undefined,
            previousSentMessage: requestCredentialMessage !== null && requestCredentialMessage !== void 0 ? requestCredentialMessage : undefined,
        });
        const credentialRequestMetadata = credentialRecord.metadata.get(repository_1.CredentialMetadataKeys.IndyRequest);
        if (!credentialRequestMetadata) {
            throw new errors_1.CredentialProblemReportError(`Missing required request metadata for credential with id ${credentialRecord.id}`, { problemCode: errors_1.CredentialProblemReportReason.IssuanceAbandoned });
        }
        const indyCredential = issueMessage.indyCredential;
        if (!indyCredential) {
            throw new errors_1.CredentialProblemReportError(`Missing required base64 or json encoded attachment data for credential with thread id ${issueMessage.threadId}`, { problemCode: errors_1.CredentialProblemReportReason.IssuanceAbandoned });
        }
        // get the revocation registry and pass it to the process (store) credential method
        const issueAttachment = issueMessage.getAttachmentById(messages_1.INDY_CREDENTIAL_ATTACHMENT_ID);
        if (!issueAttachment) {
            throw new error_1.AriesFrameworkError('Missing credential attachment in processCredential');
        }
        const options = {
            credentialAttachment: issueAttachment,
        };
        await this.formatService.processCredential(options, credentialRecord);
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.CredentialReceived);
        await this.didCommMessageRepository.saveAgentMessage({
            agentMessage: issueMessage,
            role: storage_1.DidCommMessageRole.Receiver,
            associatedRecordId: credentialRecord.id,
        });
        return credentialRecord;
    }
    /**
     * Process a received {@link CredentialAckMessage}.
     *
     * @param messageContext The message context containing a credential acknowledgement message
     * @returns credential record associated with the credential acknowledgement message
     *
     */
    /**
     * Create a {@link CredentialAckMessage} as response to a received credential.
     *
     * @param credentialRecord The credential record for which to create the credential acknowledgement
     * @returns Object containing credential acknowledgement message and associated credential record
     *
     */
    async createAck(credentialRecord) {
        credentialRecord.assertState(CredentialState_1.CredentialState.CredentialReceived);
        // Create message
        const ackMessage = new messages_1.V1CredentialAckMessage({
            status: common_1.AckStatus.OK,
            threadId: credentialRecord.threadId,
        });
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.Done);
        return { message: ackMessage, credentialRecord };
    }
    async processAck(messageContext) {
        const { message: credentialAckMessage, connection } = messageContext;
        this.logger.debug(`Processing credential ack with id ${credentialAckMessage.id}`);
        const credentialRecord = await this.getByThreadAndConnectionId(credentialAckMessage.threadId, connection === null || connection === void 0 ? void 0 : connection.id);
        const requestCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1RequestCredentialMessage,
        });
        const issueCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1IssueCredentialMessage,
        });
        // Assert
        credentialRecord.assertState(CredentialState_1.CredentialState.CredentialIssued);
        this.connectionService.assertConnectionOrServiceDecorator(messageContext, {
            previousReceivedMessage: requestCredentialMessage !== null && requestCredentialMessage !== void 0 ? requestCredentialMessage : undefined,
            previousSentMessage: issueCredentialMessage !== null && issueCredentialMessage !== void 0 ? issueCredentialMessage : undefined,
        });
        // Update record
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.Done);
        return credentialRecord;
    }
    registerHandlers() {
        this.dispatcher.registerHandler(new handlers_1.V1ProposeCredentialHandler(this, this.agentConfig, this.didCommMessageRepository));
        this.dispatcher.registerHandler(new handlers_1.V1OfferCredentialHandler(this, this.agentConfig, this.mediationRecipientService, this.didCommMessageRepository));
        this.dispatcher.registerHandler(new handlers_1.V1RequestCredentialHandler(this, this.agentConfig, this.didCommMessageRepository));
        this.dispatcher.registerHandler(new handlers_1.V1IssueCredentialHandler(this, this.agentConfig, this.didCommMessageRepository));
        this.dispatcher.registerHandler(new handlers_1.V1CredentialAckHandler(this));
        this.dispatcher.registerHandler(new handlers_1.V1CredentialProblemReportHandler(this));
        this.dispatcher.registerHandler(new handlers_1.V1RevocationNotificationHandler(this.revocationService));
    }
    /**
     *
     * Get the version of Issue Credentials according to AIP1.0 or AIP2.0
     * @returns the version of this credential service
     */
    getVersion() {
        return CredentialProtocolVersion_1.CredentialProtocolVersion.V1;
    }
    /**
     * Negotiate a credential offer as holder (by sending a credential proposal message) to the connection
     * associated with the credential record.
     *
     * @param credentialOptions configuration for the offer see {@link NegotiateProposalOptions}
     * @param credentialRecord the credential exchange record for this proposal
     * @returns Credential record associated with the credential offer and the corresponding new offer message
     *
     */
    // AUTO RESPOND METHODS
    shouldAutoRespondToCredential(credentialRecord, credentialMessage) {
        const formatService = this.getFormatService();
        let credentialAttachment;
        if (credentialMessage) {
            credentialAttachment = credentialMessage.getAttachmentById(messages_1.INDY_CREDENTIAL_ATTACHMENT_ID);
        }
        const handlerOptions = {
            credentialRecord,
            autoAcceptType: this.agentConfig.autoAcceptCredentials,
            credentialAttachment,
        };
        const shouldAutoReturn = this.agentConfig.autoAcceptCredentials === CredentialAutoAcceptType_1.AutoAcceptCredential.Always ||
            credentialRecord.autoAcceptCredential === CredentialAutoAcceptType_1.AutoAcceptCredential.Always ||
            formatService.shouldAutoRespondToCredential(handlerOptions);
        return shouldAutoReturn;
    }
    async shouldAutoRespondToProposal(handlerOptions) {
        const autoAccept = CredentialResponseCoordinator_1.CredentialResponseCoordinator.composeAutoAccept(handlerOptions.credentialRecord.autoAcceptCredential, handlerOptions.autoAcceptType);
        if (autoAccept === CredentialAutoAcceptType_1.AutoAcceptCredential.ContentApproved) {
            return (this.areProposalValuesValid(handlerOptions.credentialRecord, handlerOptions.messageAttributes) &&
                this.areProposalAndOfferDefinitionIdEqual(handlerOptions.credentialDefinitionId, handlerOptions.offerAttachment));
        }
        return false;
    }
    areProposalValuesValid(credentialRecord, proposeMessageAttributes) {
        const { credentialAttributes } = credentialRecord;
        if (proposeMessageAttributes && credentialAttributes) {
            const proposeValues = CredentialUtils_1.CredentialUtils.convertAttributesToValues(proposeMessageAttributes);
            const defaultValues = CredentialUtils_1.CredentialUtils.convertAttributesToValues(credentialAttributes);
            if (CredentialUtils_1.CredentialUtils.checkValuesMatch(proposeValues, defaultValues)) {
                return true;
            }
        }
        return false;
    }
    areProposalAndOfferDefinitionIdEqual(proposalCredentialDefinitionId, offerAttachment) {
        let credOffer;
        if (offerAttachment) {
            credOffer = offerAttachment.getDataAsJson();
        }
        const offerCredentialDefinitionId = credOffer === null || credOffer === void 0 ? void 0 : credOffer.cred_def_id;
        return proposalCredentialDefinitionId === offerCredentialDefinitionId;
    }
    shouldAutoRespondToRequest(credentialRecord, requestMessage, proposeMessage, offerMessage) {
        const formatService = this.getFormatService();
        let proposalAttachment, offerAttachment, requestAttachment;
        if (offerMessage) {
            offerAttachment = offerMessage.getAttachmentById(messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID);
        }
        if (requestMessage) {
            requestAttachment = requestMessage.getAttachmentById(messages_1.INDY_CREDENTIAL_REQUEST_ATTACHMENT_ID);
        }
        const handlerOptions = {
            credentialRecord,
            autoAcceptType: this.agentConfig.autoAcceptCredentials,
            proposalAttachment,
            offerAttachment,
            requestAttachment,
        };
        const shouldAutoReturn = this.agentConfig.autoAcceptCredentials === CredentialAutoAcceptType_1.AutoAcceptCredential.Always ||
            credentialRecord.autoAcceptCredential === CredentialAutoAcceptType_1.AutoAcceptCredential.Always ||
            formatService.shouldAutoRespondToRequest(handlerOptions);
        return shouldAutoReturn;
    }
    shouldAutoRespondToOffer(credentialRecord, offerMessage, proposeMessage) {
        var _a;
        const formatService = this.getFormatService();
        let proposalAttachment;
        const offerAttachment = offerMessage.getAttachmentById(messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID);
        if (proposeMessage && proposeMessage.appendedAttachments) {
            proposalAttachment = proposeMessage.getAttachment();
        }
        const offerValues = (_a = offerMessage.credentialPreview) === null || _a === void 0 ? void 0 : _a.attributes;
        const handlerOptions = {
            credentialRecord,
            autoAcceptType: this.agentConfig.autoAcceptCredentials,
            messageAttributes: offerValues,
            proposalAttachment,
            offerAttachment,
        };
        const shouldAutoReturn = this.agentConfig.autoAcceptCredentials === CredentialAutoAcceptType_1.AutoAcceptCredential.Always ||
            credentialRecord.autoAcceptCredential === CredentialAutoAcceptType_1.AutoAcceptCredential.Always ||
            formatService.shouldAutoRespondToProposal(handlerOptions);
        return shouldAutoReturn;
    }
    // REPOSITORY METHODS
    async getOfferMessage(id) {
        return await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: id,
            messageClass: messages_1.V1OfferCredentialMessage,
        });
    }
    async getRequestMessage(id) {
        return await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: id,
            messageClass: messages_1.V1RequestCredentialMessage,
        });
    }
    async getCredentialMessage(id) {
        return await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: id,
            messageClass: messages_1.V1IssueCredentialMessage,
        });
    }
    getFormats() {
        throw new Error('Method not implemented.');
    }
    getFormatService() {
        return this.formatService;
    }
};
V1CredentialService = __decorate([
    tsyringe_1.scoped(tsyringe_1.Lifecycle.ContainerScoped),
    __metadata("design:paramtypes", [services_1.ConnectionService,
        storage_1.DidCommMessageRepository,
        AgentConfig_1.AgentConfig,
        routing_1.MediationRecipientService,
        Dispatcher_1.Dispatcher,
        EventEmitter_1.EventEmitter,
        repository_1.CredentialRepository,
        IndyCredentialFormatService_1.IndyCredentialFormatService,
        services_2.RevocationService])
], V1CredentialService);
exports.V1CredentialService = V1CredentialService;
//# sourceMappingURL=V1CredentialService.js.map