import type { TagsBase } from '../../../storage/BaseRecord';
import type { AutoAcceptCredential } from '../CredentialAutoAcceptType';
import type { CredentialProtocolVersion } from '../CredentialProtocolVersion';
import type { CredentialState } from '../CredentialState';
import type { CredentialFormatType } from '../CredentialsModuleOptions';
import type { RevocationNotification } from '../models/RevocationNotification';
import type { CredentialMetadata } from './CredentialMetadataTypes';
import { Attachment } from '../../../decorators/attachment/Attachment';
import { BaseRecord } from '../../../storage/BaseRecord';
import { CredentialPreviewAttribute } from '../models/CredentialPreviewAttributes';
import { CredentialInfo } from '../protocol/v1/models/CredentialInfo';
export interface CredentialExchangeRecordProps {
    id?: string;
    createdAt?: Date;
    state: CredentialState;
    connectionId?: string;
    threadId: string;
    protocolVersion: CredentialProtocolVersion;
    tags?: CustomCredentialTags;
    credentialAttributes?: CredentialPreviewAttribute[];
    autoAcceptCredential?: AutoAcceptCredential;
    linkedAttachments?: Attachment[];
    revocationNotification?: RevocationNotification;
    errorMessage?: string;
    credentials?: CredentialRecordBinding[];
}
export declare type CustomCredentialTags = TagsBase;
export declare type DefaultCredentialTags = {
    threadId: string;
    connectionId?: string;
    state: CredentialState;
    credentialIds: string[];
};
export interface CredentialRecordBinding {
    credentialRecordType: CredentialFormatType;
    credentialRecordId: string;
    credentialId?: string;
    indyRevocationRegistryId?: string;
    indyCredentialRevocationId?: string;
}
export declare class CredentialExchangeRecord extends BaseRecord<DefaultCredentialTags, CustomCredentialTags, CredentialMetadata> {
    connectionId?: string;
    threadId: string;
    state: CredentialState;
    autoAcceptCredential?: AutoAcceptCredential;
    revocationNotification?: RevocationNotification;
    errorMessage?: string;
    protocolVersion: CredentialProtocolVersion;
    credentials: CredentialRecordBinding[];
    credentialAttributes?: CredentialPreviewAttribute[];
    linkedAttachments?: Attachment[];
    static readonly type = "CredentialRecord";
    readonly type = "CredentialRecord";
    constructor(props: CredentialExchangeRecordProps);
    getTags(): {
        threadId: string;
        connectionId: string | undefined;
        state: CredentialState;
        credentialIds: string[];
        indyRevocationRegistryId: string | undefined;
        indyCredentialRevocationId: string | undefined;
    };
    getCredentialInfo(): CredentialInfo | null;
    assertVersion(version: string): void;
    assertState(expectedStates: CredentialState | CredentialState[]): void;
    assertConnection(currentConnectionId: string): void;
}
