"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RevocationService = void 0;
const tsyringe_1 = require("tsyringe");
const AgentConfig_1 = require("../../../agent/AgentConfig");
const EventEmitter_1 = require("../../../agent/EventEmitter");
const AriesFrameworkError_1 = require("../../../error/AriesFrameworkError");
const CredentialEvents_1 = require("../CredentialEvents");
const RevocationNotification_1 = require("../models/RevocationNotification");
const repository_1 = require("../repository");
let RevocationService = class RevocationService {
    constructor(credentialRepository, eventEmitter, agentConfig) {
        this.credentialRepository = credentialRepository;
        this.eventEmitter = eventEmitter;
        this.logger = agentConfig.logger;
    }
    async processRevocationNotification(indyRevocationRegistryId, indyCredentialRevocationId, connection, comment) {
        const query = { indyRevocationRegistryId, indyCredentialRevocationId };
        this.logger.trace(`Getting record by query for revocation notification:`, query);
        const credentialRecord = await this.credentialRepository.getSingleByQuery(query);
        credentialRecord.assertConnection(connection.id);
        credentialRecord.revocationNotification = new RevocationNotification_1.RevocationNotification(comment);
        await this.credentialRepository.update(credentialRecord);
        this.logger.trace('Emitting RevocationNotificationReceivedEvent');
        this.eventEmitter.emit({
            type: CredentialEvents_1.CredentialEventTypes.RevocationNotificationReceived,
            payload: {
                credentialRecord,
            },
        });
    }
    /**
     * Process a received {@link V1RevocationNotificationMessage}. This will create a
     * {@link RevocationNotification} and store it in the corresponding {@link CredentialRecord}
     *
     * @param messageContext message context of RevocationNotificationMessageV1
     */
    async v1ProcessRevocationNotification(messageContext) {
        this.logger.info('Processing revocation notification v1', { message: messageContext.message });
        // ThreadID = indy::<revocation_registry_id>::<credential_revocation_id>
        const threadRegex = /(indy)::((?:[\dA-z]{21,22}):4:(?:[\dA-z]{21,22}):3:[Cc][Ll]:(?:(?:[1-9][0-9]*)|(?:[\dA-z]{21,22}:2:.+:[0-9.]+))(?::[\dA-z]+)?:CL_ACCUM:(?:[\dA-z-]+))::(\d+)$/;
        const threadId = messageContext.message.issueThread;
        try {
            const threadIdGroups = threadId.match(threadRegex);
            if (threadIdGroups) {
                const [, , indyRevocationRegistryId, indyCredentialRevocationId] = threadIdGroups;
                const comment = messageContext.message.comment;
                const connection = messageContext.assertReadyConnection();
                await this.processRevocationNotification(indyRevocationRegistryId, indyCredentialRevocationId, connection, comment);
            }
            else {
                throw new AriesFrameworkError_1.AriesFrameworkError(`Incorrect revocation notification threadId format: \n${threadId}\ndoes not match\n"indy::<revocation_registry_id>::<credential_revocation_id>"`);
            }
        }
        catch (error) {
            this.logger.warn('Failed to process revocation notification message', { error, threadId });
        }
    }
    /**
     * Process a received {@link V2RevocationNotificationMessage}. This will create a
     * {@link RevocationNotification} and store it in the corresponding {@link CredentialRecord}
     *
     * @param messageContext message context of RevocationNotificationMessageV2
     */
    async v2ProcessRevocationNotification(messageContext) {
        this.logger.info('Processing revocation notification v2', { message: messageContext.message });
        // CredentialId = <revocation_registry_id>::<credential_revocation_id>
        const credentialIdRegex = /((?:[\dA-z]{21,22}):4:(?:[\dA-z]{21,22}):3:[Cc][Ll]:(?:(?:[1-9][0-9]*)|(?:[\dA-z]{21,22}:2:.+:[0-9.]+))(?::[\dA-z]+)?:CL_ACCUM:(?:[\dA-z-]+))::(\d+)$/;
        const credentialId = messageContext.message.credentialId;
        try {
            const credentialIdGroups = credentialId.match(credentialIdRegex);
            if (credentialIdGroups) {
                const [, indyRevocationRegistryId, indyCredentialRevocationId] = credentialIdGroups;
                const comment = messageContext.message.comment;
                const connection = messageContext.assertReadyConnection();
                await this.processRevocationNotification(indyRevocationRegistryId, indyCredentialRevocationId, connection, comment);
            }
            else {
                throw new AriesFrameworkError_1.AriesFrameworkError(`Incorrect revocation notification credentialId format: \n${credentialId}\ndoes not match\n"<revocation_registry_id>::<credential_revocation_id>"`);
            }
        }
        catch (error) {
            this.logger.warn('Failed to process revocation notification message', { error, credentialId });
        }
    }
};
RevocationService = __decorate([
    tsyringe_1.scoped(tsyringe_1.Lifecycle.ContainerScoped),
    __metadata("design:paramtypes", [repository_1.CredentialRepository, EventEmitter_1.EventEmitter, AgentConfig_1.AgentConfig])
], RevocationService);
exports.RevocationService = RevocationService;
//# sourceMappingURL=RevocationService.js.map