"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DidDocument = void 0;
const class_transformer_1 = require("class-transformer");
const class_validator_1 = require("class-validator");
const JsonTransformer_1 = require("../../../utils/JsonTransformer");
const transformers_1 = require("../../../utils/transformers");
const service_1 = require("./service");
const verificationMethod_1 = require("./verificationMethod");
class DidDocument {
    constructor(options) {
        var _a;
        this.context = ['https://w3id.org/did/v1'];
        if (options) {
            this.context = (_a = options.context) !== null && _a !== void 0 ? _a : this.context;
            this.id = options.id;
            this.alsoKnownAs = options.alsoKnownAs;
            this.controller = options.controller;
            this.verificationMethod = options.verificationMethod;
            this.service = options.service;
            this.authentication = options.authentication;
            this.assertionMethod = options.assertionMethod;
            this.keyAgreement = options.keyAgreement;
            this.capabilityInvocation = options.capabilityInvocation;
            this.capabilityDelegation = options.capabilityDelegation;
        }
    }
    dereferenceKey(keyId) {
        var _a;
        // TODO: once we use JSON-LD we should use that to resolve references in did documents.
        // for now we check whether the key id ends with the keyId.
        // so if looking for #123 and key.id is did:key:123#123, it is valid. But #123 as key.id is also valid
        const verificationMethod = (_a = this.verificationMethod) === null || _a === void 0 ? void 0 : _a.find((key) => key.id.endsWith(keyId));
        if (!verificationMethod) {
            throw new Error(`Unable to locate verification with id '${keyId}'`);
        }
        return verificationMethod;
    }
    /**
     * Returns all of the service endpoints matching the given type.
     *
     * @param type The type of service(s) to query.
     */
    getServicesByType(type) {
        var _a, _b;
        return ((_b = (_a = this.service) === null || _a === void 0 ? void 0 : _a.filter((service) => service.type === type)) !== null && _b !== void 0 ? _b : []);
    }
    /**
     * Returns all of the service endpoints matching the given class
     *
     * @param classType The class to query services.
     */
    getServicesByClassType(classType) {
        var _a, _b;
        return ((_b = (_a = this.service) === null || _a === void 0 ? void 0 : _a.filter((service) => service instanceof classType)) !== null && _b !== void 0 ? _b : []);
    }
    /**
     * Get all DIDComm services ordered by priority descending. This means the highest
     * priority will be the first entry.
     */
    get didCommServices() {
        var _a, _b;
        const didCommServiceTypes = [service_1.IndyAgentService.type, service_1.DidCommService.type];
        const services = ((_b = (_a = this.service) === null || _a === void 0 ? void 0 : _a.filter((service) => didCommServiceTypes.includes(service.type))) !== null && _b !== void 0 ? _b : []);
        // Sort services based on indicated priority
        return services.sort((a, b) => b.priority - a.priority);
    }
    get recipientKeys() {
        // Get a `recipientKeys` entries from the did document
        return this.didCommServices.reduce((recipientKeys, service) => recipientKeys.concat(service.recipientKeys), []);
    }
    toJSON() {
        return JsonTransformer_1.JsonTransformer.toJSON(this);
    }
}
__decorate([
    class_transformer_1.Expose({ name: '@context' }),
    transformers_1.IsStringOrStringArray(),
    __metadata("design:type", Object)
], DidDocument.prototype, "context", void 0);
__decorate([
    class_validator_1.IsString(),
    __metadata("design:type", String)
], DidDocument.prototype, "id", void 0);
__decorate([
    class_validator_1.IsArray(),
    class_validator_1.IsString({ each: true }),
    class_validator_1.IsOptional(),
    __metadata("design:type", Array)
], DidDocument.prototype, "alsoKnownAs", void 0);
__decorate([
    transformers_1.IsStringOrStringArray(),
    class_validator_1.IsOptional(),
    __metadata("design:type", Object)
], DidDocument.prototype, "controller", void 0);
__decorate([
    class_validator_1.IsArray(),
    class_validator_1.ValidateNested({ each: true }),
    class_transformer_1.Type(() => verificationMethod_1.VerificationMethod),
    class_validator_1.IsOptional(),
    __metadata("design:type", Array)
], DidDocument.prototype, "verificationMethod", void 0);
__decorate([
    class_validator_1.IsArray(),
    service_1.ServiceTransformer(),
    class_validator_1.IsOptional(),
    __metadata("design:type", Array)
], DidDocument.prototype, "service", void 0);
__decorate([
    class_validator_1.IsArray(),
    verificationMethod_1.VerificationMethodTransformer(),
    verificationMethod_1.IsStringOrVerificationMethod({ each: true }),
    class_validator_1.IsOptional(),
    __metadata("design:type", Array)
], DidDocument.prototype, "authentication", void 0);
__decorate([
    class_validator_1.IsArray(),
    verificationMethod_1.VerificationMethodTransformer(),
    verificationMethod_1.IsStringOrVerificationMethod({ each: true }),
    class_validator_1.IsOptional(),
    __metadata("design:type", Array)
], DidDocument.prototype, "assertionMethod", void 0);
__decorate([
    class_validator_1.IsArray(),
    verificationMethod_1.VerificationMethodTransformer(),
    verificationMethod_1.IsStringOrVerificationMethod({ each: true }),
    class_validator_1.IsOptional(),
    __metadata("design:type", Array)
], DidDocument.prototype, "keyAgreement", void 0);
__decorate([
    class_validator_1.IsArray(),
    verificationMethod_1.VerificationMethodTransformer(),
    verificationMethod_1.IsStringOrVerificationMethod({ each: true }),
    class_validator_1.IsOptional(),
    __metadata("design:type", Array)
], DidDocument.prototype, "capabilityInvocation", void 0);
__decorate([
    class_validator_1.IsArray(),
    verificationMethod_1.VerificationMethodTransformer(),
    verificationMethod_1.IsStringOrVerificationMethod({ each: true }),
    class_validator_1.IsOptional(),
    __metadata("design:type", Array)
], DidDocument.prototype, "capabilityDelegation", void 0);
exports.DidDocument = DidDocument;
//# sourceMappingURL=DidDocument.js.map